%{
/*
 * Copyright (c) 2019 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <stdio.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_CC_PARSER_L_L
#include "osm_cc_parser_y.h"
extern char *__cc_parser_yylval;
extern char *trim_whitespace(char *str);
#define SAVE_POS save_pos()
static void save_pos();

extern int column_num;
extern int line_num;

%}


%option nounput noinput
STRING [a-zA-Z0-9'.''_''\-''|''&''@''('')''/''#''!''?''+''=''[''\]''\/''{''}'';''%''*''^''\\'':']*

%%

\#.*\n                                       { SAVE_POS; return COMMENT; }
[ \t]*\n                       	             { SAVE_POS; } /* ignore end of line and empty lines*/
[' ']*                                       { SAVE_POS; } /* ignore spaces */
,                                            { SAVE_POS; return COMMA; }
[ \t]*profile_start[ \t]*      	             { SAVE_POS; return PROFILE_START; }
[ \t]*profile_end[ \t]*        	             { SAVE_POS; return PROFILE_END; }
[ \t]*profiles_start[ \t]*     	             { SAVE_POS; return PROFILES_START; }
[ \t]*profiles_end[ \t]*       	             { SAVE_POS; return PROFILES_END; }
[ \t]*sl_config_start[ \t]*    	             { SAVE_POS; return SL_CONFIG_START; }
[ \t]*sl_config_end[ \t]*      	             { SAVE_POS; return SL_CONFIG_END; }
[ \t]*vl_config_start[ \t]*    	             { SAVE_POS; return VL_CONFIG_START; }
[ \t]*vl_config_end[ \t]*      	             { SAVE_POS; return VL_CONFIG_END; }
[ \t]*ca_config_start[ \t]*    	             { SAVE_POS; return CA_CONFIG_START; }
[ \t]*ca_config_end[ \t]*      	             { SAVE_POS; return CA_CONFIG_END; }
[ \t]*switch_config_start[ \t]*	             { SAVE_POS; return SWITCH_CONFIG_START; }
[ \t]*switch_config_end[ \t]*  	             { SAVE_POS; return SWITCH_CONFIG_END; }
[ \t]*name:                    	             { SAVE_POS; return NAME; }
[ \t]*mode:                    	             { SAVE_POS; return MODE; }
[ \t]*percent:                 	             { SAVE_POS; return PERCENT; }
[ \t]*min:                     	             { SAVE_POS; return MIN; }
[ \t]*max:                     	             { SAVE_POS; return MAX; }
[ \t]*group:                                 { SAVE_POS; return GROUP; }
[ \t]*sl_profiles:             	             { SAVE_POS; return SL_PROFILES; }
[ \t]*vl_profiles:             	             { SAVE_POS; return VL_PROFILES; }
[ \t]*enable:                  	             { SAVE_POS; return ENABLE; }
[ \t]*aqs_weight:              	             { SAVE_POS; return AQS_WEIGHT; }
[ \t]*aqs_time:                              { SAVE_POS; return AQS_TIME; }
[ \t]*enable_notify:                         { SAVE_POS; return ENABLE_NOTIFY; }
[ \t]*enable_react:                          { SAVE_POS; return ENABLE_REACT; }
[ \t]*enable_clamp_tgt_rate:                 { SAVE_POS; return ENABLE_CLAMP_TGT_RATE; }
[ \t]*enable_clamp_tgt_rate_after_time_inc:  { SAVE_POS; return ENABLE_CLAMP_TGT_RATE_AFTER_TIME_INC; }
[ \t]*rpg_time_reset:                        { SAVE_POS; return RPG_TIME_RESET; }
[ \t]*rpg_byte_reset:                        { SAVE_POS; return RPG_BYTE_RESET; }
[ \t]*rpg_threshold:                         { SAVE_POS; return RPG_THRESHOLD; }
[ \t]*rpg_max_rate:                          { SAVE_POS; return RPG_MAX_RATE; }
[ \t]*rpg_ai_rate:                           { SAVE_POS; return RPG_AI_RATE; }
[ \t]*rpg_hai_rate:                          { SAVE_POS; return RPG_HAI_RATE; }
[ \t]*rpg_gd:                                { SAVE_POS; return RPG_GD; }
[ \t]*rpg_min_dec_fac:                       { SAVE_POS; return RPG_MIN_DEC_FAC; }
[ \t]*rpg_min_rate:                          { SAVE_POS; return RPG_MIN_RATE; }
[ \t]*rate_to_set_on_first_cnp:              { SAVE_POS; return RATE_TO_SET_ON_FIRST_CNP; }
[ \t]*dce_tcp_g:                             { SAVE_POS; return DCE_TCP_G; }
[ \t]*dce_tcp_rtt:                           { SAVE_POS; return DCE_TCP_RTT; }
[ \t]*rate_reduce_monitor_period:            { SAVE_POS; return RATE_REDUCE_MONITOR_PERIOD; }
[ \t]*initial_alpha_value:                   { SAVE_POS; return INITIAL_ALPHA_VALUE; }
[ \t]*min_time_between_cnps:                 { SAVE_POS; return MIN_TIME_BETWEEN_CNPS; }
[ \t]*cnp_sl_mode:                           { SAVE_POS; return CNP_SL_MODE; }
[ \t]*cnp_sl:                                { SAVE_POS; return CNP_SL; }
0[xX][0-9a-fA-F]+ {
			free(__cc_parser_yylval);
			__cc_parser_yylval = strdup(yytext);
			SAVE_POS;
			return EXNUMBER;
		  }
[0-9]+	{
		free(__cc_parser_yylval);
		__cc_parser_yylval = strdup(yytext);
		SAVE_POS;
		return NUMBER;
        }
{STRING} {
		free(__cc_parser_yylval);
		__cc_parser_yylval=strdup(trim_whitespace(yytext));
		SAVE_POS;
		return WORD;
	}
%%


static void save_pos()
{
	int i;
	for (i = 0; i < yyleng; i++) {
		if (yytext[i] == '\n') {
			line_num++;
			column_num = 1;
		} else {
			column_num++;
		}
	}
}
