%{
/*
 * Copyright (c) 2019 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_CC_PARSER_Y_Y
#include <opensm/osm_opensm.h>

#define CC_MAX_MODE_LEN 1
#define NOT_DEFINED 0xff
#define YYSTYPE char*

typedef struct tmp_parser_struct_ {
	cl_list_t  str_list;
	cl_list_t  num_list;
} tmp_parser_struct_t;

extern FILE * __cc_parser_in;
static boolean_t in_profile;
static boolean_t in_sl_config;
static char *profile_min = NULL;
static char *profile_max = NULL;
static char *profile_percent = NULL;
static struct osm_opensm *p_osm;
cl_list_t num_list;
cl_list_t str_list;
char *__cc_parser_yylval = NULL;

tmp_parser_struct_t tmp_parser_struct;

int column_num;
int line_num;

osm_cc_profiles_conf_t *p_current_profiles_conf;
osm_cc_sl_conf_t *p_current_sl_conf;
osm_cc_vl_conf_t *p_current_vl_conf;
osm_cc_sw_conf_t *p_current_sw_conf;
osm_cc_ca_conf_t *p_current_ca_conf;
int current_profile_index = 0;

extern char *trim_whitespace(char *str);

static void clear_val(char **c)
{
	free(*c);
	*c = NULL;
}

static void yyerror(const char *format, ...)
{
    char s[256];
    va_list pvar;

    OSM_LOG_ENTER(&p_osm->log);

    va_start(pvar, format);
    vsnprintf(s, sizeof(s), format, pvar);
    va_end(pvar);

    OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR AC05: "
            "Syntax error (line %d:%d): %s\n",
            line_num, column_num, s);
    fprintf(stderr, "Error in CC Policy File (line %d:%d): %s.\n",
            line_num, column_num, s);
    OSM_LOG_EXIT(&p_osm->log);
}


/*void osm_destroy_profile_list(cl_qlist_t *p_list)
{
	cl_list_item_t *p_item;

	if (!p_list)
		return;

	p_item = cl_qlist_remove_head(p_list);
	while (p_item != cl_qlist_end(p_list)) {
		osm_destroy_cc_profile((osm_cc_profile_t *)p_item);
		p_item = cl_qlist_remove_head(p_list);
	}
}*/

static osm_cc_profiles_conf_t* create_profiles_conf()
{
	osm_cc_profiles_conf_t *p_profiles_conf = NULL;

	p_profiles_conf = calloc(1, sizeof(osm_cc_profiles_conf_t));
	if (!p_profiles_conf) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E06: "
			"Failed allocating memory for new profiles conf\n");
		goto Exit;
	}

	p_profiles_conf->mode = NOT_DEFINED;

	//cl_qlist_insert_tail(&p_osm->sm.cc_mgr.cc_profiles, &p_profiles_conf->list_item);
Exit:
	return p_profiles_conf;
}

static osm_cc_ca_conf_t* create_ca_conf()
{
	osm_cc_ca_conf_t *p_ca_conf = NULL;

	p_ca_conf = calloc(1, sizeof(osm_cc_ca_conf_t));
	if (!p_ca_conf) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E06: "
			"Failed allocating memory for new CA conf\n");
		goto Exit;
	}

	cl_qlist_insert_tail(&p_osm->sm.cc_mgr.ca_conf, &p_ca_conf->list_item);
Exit:
	return p_ca_conf;
}

static boolean_t create_sl_conf()
{
	boolean_t res = TRUE;

	p_current_sl_conf = calloc(1, sizeof(osm_cc_sl_conf_t));
	if (!p_current_sl_conf) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E06: "
			"Failed allocating memory for new SL profiles conf\n");
		res = FALSE;
		goto Exit;
	}

	cl_qlist_insert_tail(&p_osm->sm.cc_mgr.sl_profiles_conf, &p_current_sl_conf->list_item);
Exit:
	return res;
}

static boolean_t create_vl_conf()
{
	boolean_t res = TRUE;

	p_current_vl_conf = calloc(1, sizeof(osm_cc_vl_conf_t));
	if (!p_current_vl_conf) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E06: "
			"Failed allocating memory for new VL profiles conf\n");
		res = FALSE;
		goto Exit;
	}

	cl_qlist_insert_tail(&p_osm->sm.cc_mgr.vl_profiles_conf, &p_current_vl_conf->list_item);
Exit:
	return res;
}

static boolean_t create_sw_conf()
{
	boolean_t res = TRUE;

	p_current_sw_conf = calloc(1, sizeof(osm_cc_sw_conf_t));
	if (!p_current_sw_conf) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E06: "
			"Failed allocating memory for new SW conf\n");
		res = FALSE;
		goto Exit;
	}

	cl_qlist_insert_tail(&p_osm->sm.cc_mgr.sw_conf, &p_current_sw_conf->list_item);
Exit:
	return res;
}

static boolean_t create_profile()
{
	osm_cc_profile_t *p_profile;
	boolean_t res = TRUE;

	if (current_profile_index >= OSM_CC_MAX_PROFILE_NUM) {
		res = FALSE;
		goto Exit;
	}

	p_profile = &p_current_profiles_conf->profiles[current_profile_index];

// TODO: validate min, max, percent
	p_profile->min = atoi(profile_min);
	p_profile->max = atoi(profile_max);
	p_profile->percent = atoi(profile_percent);

	current_profile_index++;

Exit:
	return res;
}

static void clear_profile()
{
	clear_val(&profile_min);
	clear_val(&profile_max);
	clear_val(&profile_percent);
	clear_val(&__cc_parser_yylval);
}

boolean_t __cc_profile_is_tag_defined(const char *str, const char *tag_name)
{
	if (str) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E01: Profile %s "
			"qualifier already defined (%.200s), Aborting "
			"congestion control parsing!!!\n", tag_name, str);
		return TRUE;
	}

	return FALSE;
}

/*void __cc_parser_error(const char *str)
{
        OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "%.200s\n", str);
}*/

int __cc_parser_wrap()
{
        return 1;
}

int __cc_parser_parse();
int __cc_parser_lex();

int parse_congestion_control_policy(struct osm_opensm *osm_ptr)
{
	int res = 0;
	p_osm = osm_ptr;
	__cc_parser_in = NULL;
	cl_list_construct(&num_list);
	cl_list_init(&num_list, 10);
	cl_list_construct(&str_list);
	cl_list_init(&str_list, 10);
	current_profile_index = 0;
	line_num = 0;
	column_num = 0;

	if (!p_osm->subn.opt.congestion_control_policy_file) {
		OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "No policy file to parse -"
			" Quit cc parser\n");
		res = 1;
		goto Exit;
	}

	osm_cc_mgr_destroy(&p_osm->sm.cc_mgr);
	__cc_parser_in = fopen(p_osm->subn.opt.congestion_control_policy_file, "r");
	if (!__cc_parser_in) {
		__cc_parser_error("ERR 8E02: Congestion control policy file specified but not found\n");
		res = 1;
		goto Exit;
	}

        res =  __cc_parser_parse();
	if (res) {
		__cc_parser_error("ERR 8E04: Congestion control policy file parsing failed\n");
		osm_cc_mgr_destroy(&p_osm->sm.cc_mgr);
	} else {
		OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "Congestion control "
			"policy file parsed successfully\n");
	}

Exit:
	if (__cc_parser_in) {
		fclose(__cc_parser_in);
		__cc_parser_in = NULL;
	}
	clear_profile();
	return res;
}

%}

%token COMMENT
%token COMMA
%token WORD
%token GROUP
%token EXNUMBER
%token NUMBER
%token NAME

%token PROFILES_START
%token PROFILES_END
%token MODE

%token PROFILE_START
%token PROFILE_END
%token PERCENT
%token MIN
%token MAX

%token SL_CONFIG_START
%token SL_CONFIG_END
%token SL_PROFILES

%token VL_CONFIG_START
%token VL_CONFIG_END
%token VL_PROFILES

%token SWITCH_CONFIG_START
%token SWITCH_CONFIG_END
%token ENABLE
%token AQS_WEIGHT
%token AQS_TIME

%token CA_CONFIG_START
%token CA_CONFIG_END
%token ENABLE_NOTIFY
%token ENABLE_REACT
%token ENABLE_CLAMP_TGT_RATE
%token ENABLE_CLAMP_TGT_RATE_AFTER_TIME_INC
%token RPG_TIME_RESET
%token RPG_BYTE_RESET
%token RPG_THRESHOLD
%token RPG_MAX_RATE
%token RPG_AI_RATE
%token RPG_HAI_RATE
%token RPG_GD
%token RPG_MIN_DEC_FAC
%token RPG_MIN_RATE
%token RATE_TO_SET_ON_FIRST_CNP
%token DCE_TCP_G
%token DCE_TCP_RTT
%token RATE_REDUCE_MONITOR_PERIOD
%token INITIAL_ALPHA_VALUE
%token RP_FLOW_DEFINITION_MODE

%token NP_FLOW_DEFINITION_MODE
%token MIN_TIME_BETWEEN_CNPS
%token CNP_SL_MODE
%token CNP_SL
%%

cc_policy: configurations;

configurations: /* empty */
		| configurations configuration
		;

comment: COMMENT {};

configuration:    profiles
		| ca_config
		| switch_config
		| vl_config
		| sl_config
		| word
		| comment
		;


ca_config: ca_config_start ca_config_entries ca_config_end
	   ;

ca_config_entries:   /* empty */
		   | ca_config_entries ca_config_entrie
		   ;

ca_config_entrie:   ca_config_group
		  | ca_config_enable_notify
		  | ca_config_enable_react
		  | ca_config_enable_clamp_tgt_rate
		  | ca_config_enable_clamp_tgt_rate_after_time_inc
		  | ca_config_rpg_time_reset
		  | ca_config_rpg_byte_reset
		  | ca_config_rpg_threshold
		  | ca_config_rpg_max_rate
		  | ca_config_rpg_ai_rate
		  | ca_config_rpg_hai_rate
		  | ca_config_rpg_gd
		  | ca_config_rpg_min_dec_fac
		  | ca_config_rpg_min_rate
		  | ca_config_rate_to_set_on_first_cnp
		  | ca_config_dce_tcp_g
		  | ca_config_dce_tcp_rtt
		  | ca_config_rate_reduce_monitor_period
		  | ca_config_initial_alpha_value
		  | ca_config_min_time_between_cnps
		  | ca_config_cnp_sl_mode
		  | ca_config_cnp_sl
		  | comment
		  ;

ca_config_start: CA_CONFIG_START {
	free(p_current_ca_conf);
	p_current_ca_conf = create_ca_conf();
	if (!p_current_ca_conf)
		return 1;
};

ca_config_end: CA_CONFIG_END {
	p_current_ca_conf = NULL;
};

ca_config_group: GROUP WORD {
	p_current_ca_conf->group_type = osm_cc_get_group_type(__cc_parser_yylval);
};

ca_config_enable_notify: ENABLE_NOTIFY NUMBER {
	p_current_ca_conf->enable_notify = atoi(__cc_parser_yylval);
};

ca_config_enable_react: ENABLE_REACT NUMBER {
	p_current_ca_conf->enable_react = atoi(__cc_parser_yylval);
};

ca_config_enable_clamp_tgt_rate: ENABLE_CLAMP_TGT_RATE NUMBER {
	p_current_ca_conf->hca_rp_params.enable_clamp_tgt_rate = atoi(__cc_parser_yylval);
};

ca_config_enable_clamp_tgt_rate_after_time_inc: ENABLE_CLAMP_TGT_RATE_AFTER_TIME_INC NUMBER {
	p_current_ca_conf->hca_rp_params.enable_clamp_tgt_rate_after_time_inc = atoi(__cc_parser_yylval);
};

ca_config_rpg_time_reset: RPG_TIME_RESET NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_time_reset = atoi(__cc_parser_yylval);
};

ca_config_rpg_byte_reset: RPG_BYTE_RESET NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_byte_reset = atoi(__cc_parser_yylval);
};

ca_config_rpg_threshold: RPG_THRESHOLD NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_threshold = atoi(__cc_parser_yylval);
};

ca_config_rpg_max_rate:	RPG_MAX_RATE NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_max_rate = atoi(__cc_parser_yylval);
};

ca_config_rpg_ai_rate: RPG_AI_RATE NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_ai_rate = atoi(__cc_parser_yylval);
};

ca_config_rpg_hai_rate: RPG_HAI_RATE NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_hai_rate = atoi(__cc_parser_yylval);
};

ca_config_rpg_gd: RPG_GD NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_gd = atoi(__cc_parser_yylval);
};

ca_config_rpg_min_dec_fac: RPG_MIN_DEC_FAC NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_min_dec_fac = atoi(__cc_parser_yylval);
};

ca_config_rpg_min_rate:	RPG_MIN_RATE NUMBER {
	p_current_ca_conf->hca_rp_params.rpg_min_rate = atoi(__cc_parser_yylval);
};

ca_config_rate_to_set_on_first_cnp: RATE_TO_SET_ON_FIRST_CNP NUMBER {
	p_current_ca_conf->hca_rp_params.rate_to_set_on_first_cnp = atoi(__cc_parser_yylval);
};

ca_config_dce_tcp_g: DCE_TCP_G NUMBER {
	p_current_ca_conf->hca_rp_params.dce_tcp_g = atoi(__cc_parser_yylval);
};

ca_config_dce_tcp_rtt: DCE_TCP_RTT NUMBER {
	p_current_ca_conf->hca_rp_params.dce_tcp_rtt = atoi(__cc_parser_yylval);
};

ca_config_rate_reduce_monitor_period: RATE_REDUCE_MONITOR_PERIOD NUMBER {
	p_current_ca_conf->hca_rp_params.rate_reduce_monitor_period = atoi(__cc_parser_yylval);
};

ca_config_initial_alpha_value: INITIAL_ALPHA_VALUE NUMBER {
	p_current_ca_conf->hca_rp_params.initial_alpha_value = atoi(__cc_parser_yylval);
};

ca_config_min_time_between_cnps: MIN_TIME_BETWEEN_CNPS NUMBER {
	p_current_ca_conf->hca_np_params.min_time_between_cnps = atoi(__cc_parser_yylval);
};

ca_config_cnp_sl_mode: CNP_SL_MODE NUMBER {
	p_current_ca_conf->hca_np_params.cnp_sl_mode = atoi(__cc_parser_yylval);
};

ca_config_cnp_sl: CNP_SL NUMBER {
	p_current_ca_conf->hca_np_params.cnp_sl = atoi(__cc_parser_yylval);
};

switch_config: switch_config_start switch_config_entries switch_config_end
	       ;

switch_config_entries:    /* empty */
			| switch_config_entries switch_config_entrie
		        ;

switch_config_entrie:	  switch_config_group
			| switch_config_enable
			| switch_config_aqs_weight
			| switch_config_aqs_time
			| comment
			;

switch_config_start: SWITCH_CONFIG_START {
	create_sw_conf();
};

switch_config_end: SWITCH_CONFIG_END {
};

switch_config_group: GROUP WORD {
	p_current_sw_conf->group_type = osm_cc_get_group_type(__cc_parser_yylval);
};

switch_config_enable: ENABLE NUMBER {
	p_current_sw_conf->enable = atoi(__cc_parser_yylval);
};

switch_config_aqs_weight: AQS_WEIGHT NUMBER {
	p_current_sw_conf->aqs_weight = atoi(__cc_parser_yylval);
};

switch_config_aqs_time: AQS_TIME NUMBER {
	p_current_sw_conf->aqs_time = atoi(__cc_parser_yylval);
};

profiles: profiles_start profiles_entries profiles_end
	  ;

profiles_entries:   /* empty */
		  | profiles_entries profiles_entrie
		  ;

profiles_entrie:   profiles_name
		 | profiles_mode
		 | profile
		 | comment
		 ;


profiles_start: PROFILES_START {
	free(p_current_profiles_conf);
	p_current_profiles_conf = create_profiles_conf();
	if (!p_current_profiles_conf)
		return 1;
}

profiles_end: PROFILES_END {
	cl_fmap_insert(&p_osm->sm.cc_mgr.cc_profiles_map,
		       &p_current_profiles_conf->name,
		       &p_current_profiles_conf->fmap_item);
	p_current_profiles_conf = NULL;
}

profile: profile_start profile_entries profile_end
	 ;

profile_entries: /* empty */
		 | profile_entries profile_entrie
		 ;

profile_entrie:   profile_percent
		| profile_min
		| profile_max
		| comment
		;

sl_config: sl_config_start sl_config_entries sl_config_end
	   ;

sl_config_start: SL_CONFIG_START {
	in_sl_config = TRUE;
	create_sl_conf();
};

sl_config_end: SL_CONFIG_END {
	in_sl_config = FALSE;
};

sl_config_entries:   /* empty */
		   | sl_config_entries sl_config_entrie
		   ;

sl_config_entrie:   sl_config_group
		  | sl_config_profile_nums
		  | comment
		  ;

sl_config_group: GROUP WORD {
	p_current_sl_conf->group_type = osm_cc_get_group_type(__cc_parser_yylval);
};

sl_config_profile_nums: SL_PROFILES num_list /* list of profile numbers */{
	uint32_t elements_num = cl_list_count(&num_list);
	cl_list_iterator_t list_iterator;
	uint64_t *p_num;
	uint64_t num = 0;
	int i = 0;

	if ((elements_num != 1) && (elements_num != IB_NUMBER_OF_SLS)) {
		yyerror("Number of SLs(%u) is incorrect, supposed to be %u",
			elements_num, IB_NUMBER_OF_SLS);
		cl_list_remove_all(&num_list);
		return 1;
	}

	list_iterator = cl_list_head(&num_list);
	while (list_iterator != cl_list_end(&num_list)) {
		p_num = (uint64_t*)cl_list_obj(list_iterator);
		num = *p_num;
		free(p_num);
		if (num > OSM_CC_MAX_PROFILE_NUM) {
			yyerror("Wrong profile number %u, "
				"value should be between 1 to %u",
				num, OSM_CC_MAX_PROFILE_NUM);
			cl_list_remove_all(&num_list);
			return 1;
		}

		p_current_sl_conf->sl2profile_num[i++] = (uint8_t)num;
		list_iterator = cl_list_next(list_iterator);
	}

	if (elements_num == 1) {
		for (; i < IB_NUMBER_OF_SLS; i++)
			p_current_sl_conf->sl2profile_num[i] = (uint8_t)num;
	}

	cl_list_remove_all(&num_list);
};

vl_config: vl_config_start vl_config_entries vl_config_end
	   ;

vl_config_start: VL_CONFIG_START {
	create_vl_conf();
};

vl_config_end: VL_CONFIG_END {
};

vl_config_entries:   /* empty */
		   | vl_config_entries vl_config_entrie
		   ;

vl_config_entrie:   vl_config_group
		  | vl_config_profiles
		  | comment
		  ;

vl_config_group: GROUP WORD {
	p_current_vl_conf->group_type = osm_cc_get_group_type(__cc_parser_yylval);
};

vl_config_profiles: VL_PROFILES string_list {
	uint32_t elements_num = cl_list_count(&str_list);
	cl_list_iterator_t list_iterator;
	osm_cc_profiles_conf_t *p_profiles_conf = NULL;
	cl_fmap_item_t *p_item;
	char *tmp_str;
	int i = 0;

	if ((elements_num != 1) && (elements_num != IB_MAX_NUM_VLS)) {
		yyerror("Number of VLs(%u) is incorrect, supposed to be %u",
			elements_num, IB_MAX_NUM_VLS);
		cl_list_remove_all(&str_list);
		return 1;
	}

	list_iterator = cl_list_head(&str_list);
	while (list_iterator != cl_list_end(&str_list)) {
		tmp_str = (char *)cl_list_obj(list_iterator);
		p_item = cl_fmap_get(&p_osm->sm.cc_mgr.cc_profiles_map, tmp_str);

		if (p_item == cl_fmap_end(&p_osm->sm.cc_mgr.cc_profiles_map)) {
			yyerror("Profiles triplet with the name: %s is not defined", tmp_str);
			return 1;
		}

		p_profiles_conf = (osm_cc_profiles_conf_t *)p_item;
		p_current_vl_conf->vl_to_profiles[i++] = p_profiles_conf;
		list_iterator = cl_list_next(list_iterator);
	}

	if (!p_profiles_conf) {
		yyerror("invalid vl profiles configuration");
		return 1;
	}

	if (elements_num == 1) {
		for (; i < IB_MAX_NUM_VLS; i++)
			p_current_vl_conf->vl_to_profiles[i] = p_profiles_conf;
	}

	cl_list_remove_all(&str_list);
};

num_list:   number
	  | num_list COMMA number
	  ;

number: NUMBER {
	uint64_t *p_num = (uint64_t*)malloc(sizeof(uint64_t));
	if (!p_num) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR,
			"ERR 8E05: Couldn't allocate memory, Aborting parsing\n");
		return 1;
	}

	*p_num = strtoull(__cc_parser_yylval, NULL, 0);
	cl_list_insert_tail(&num_list, p_num);
};

string_list:   string
	     | string_list COMMA string
	     ;

string: WORD {
	char *tmp_str = strdup(__cc_parser_yylval);
	if (!tmp_str) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR,
			"ERR 8E05: Couldn't allocate memory for "
			"profile triplet name, Aborting parsing\n");
		return 1;
	}

	cl_list_insert_tail(&str_list, tmp_str);
};

word: WORD {
	OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E05: Unexpected "
		"input |%.200s|, Aborting parsing!\n", __cc_parser_yylval);
	return 1;
};

profile_start: PROFILE_START {
	clear_profile();
	in_profile = TRUE;
};

profiles_name: NAME WORD {
	if (!p_current_profiles_conf)
		return 1;

	if (strlen(p_current_profiles_conf->name)) {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E01: Profiles conf name "
			"qualifier already defined (%.200s), Aborting "
			"congestion control parsing!!!\n", __cc_parser_yylval);
		return 1;
	}

	snprintf(p_current_profiles_conf->name, sizeof(p_current_profiles_conf->name), "%s", __cc_parser_yylval);
};

profiles_mode: MODE NUMBER {
	if (!p_current_profiles_conf)
		return 1;

	if (p_current_profiles_conf->mode != NOT_DEFINED) {
		yyerror("Profiles conf mode tag already defined");
		return 1;
	}

	if (strlen(__cc_parser_yylval) <= CC_MAX_MODE_LEN) {
		p_current_profiles_conf->mode = atoi(__cc_parser_yylval);
	} else {
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 8E07: Illegal "
			"mode, mode qualifier length must be up to "
			"%d, Aborting congestion control parsing!!!\n",
			CC_MAX_MODE_LEN);
		return 1;
	}
};

profile_percent: PERCENT NUMBER {
	if (__cc_profile_is_tag_defined(profile_percent, "profile_percent"))
		return 1;

	profile_percent = strdup(__cc_parser_yylval);
};

profile_min: MIN NUMBER {
	if (__cc_profile_is_tag_defined(profile_min, "profile_min"))
		return 1;

	profile_min = strdup(__cc_parser_yylval);
};

profile_max: MAX NUMBER {
	if (__cc_profile_is_tag_defined(profile_max, "profile_max"))
		return 1;

	profile_max = strdup(__cc_parser_yylval);
};

profile_end: PROFILE_END {
	in_profile = FALSE;
	if (!profile_max || !profile_min || !profile_percent) {
		clear_profile();
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR,
			"ERR 8E08: One of the profile tags is not defined"
			", Aborting congestion control parsing!!!\n");
		return 1;
	}

	create_profile();
	clear_profile();
};
