/*
 * Copyright (c) 2018-2020 Mellanox Technologies LTD. ALL RIGHTS RESERVED.
 * See file LICENSE for terms.
 */

#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_PARALLEL_PORT_GROUPS_CALCULATOR_CPP
#include "osm_parallel_port_groups_calculator.hpp"
#include "osm_armgr.hpp"
#include "osm_ar_algorithm.hpp"

OSMParallelPortGroupsCalculator::~OSMParallelPortGroupsCalculator()
{
	WaitForTasks();

	OSM_AR_LOG(m_osm_log_, OSM_LOG_DEBUG,
		   "OSMParallelPortGroupsCalculator destroyed\n");
}

void
OSMParallelPortGroupsCalculator::CalculatePortGroupsTree(
	uint8_t max_rank,
	u_int16_t hca_to_sw_lid_mapping[],
	uint8_t sw_lid_to_rank_mapping[])
{
	m_max_rank_ = max_rank;
	m_hca_to_sw_lid_mapping_ = hca_to_sw_lid_mapping;
	m_sw_lid_to_rank_mapping_ = sw_lid_to_rank_mapping;

	/*
	 * Go through all the switches and create their port groups.
	 * In this algorithm ports groups are created out of minimal hops
	 * links - each lid will recieve a group containing all the ports
	 * with minimal hops to this lid.
	 * So here's what we're doing here:
	 *   - for each switch
	 *       - for each port of the switch (port_num)
	 *           - calculate group bitmask for this lid
	 *           - asisign groups acording to device limitations
	 */
	map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it =
	m_sw_db_->m_sw_map.begin();
	for (; sw_node_it != m_sw_db_->m_sw_map.end(); ++sw_node_it) {

		OSM_AR_LOG(m_osm_log_, OSM_LOG_DEBUG,
			   "OSMParallelPortGroupsCalculator AddTask \n");

		AddTaskToThreadPool(
			m_thread_pool_,
			&sw_node_it->second.m_calculate_port_groups_tree_task);
	}

	WaitForTasks();

	if (!osm_log_is_active(m_osm_log_,OSM_LOG_ROUTING))
		return;

	sw_node_it = m_sw_db_->m_sw_map.begin();
	for (; sw_node_it != m_sw_db_->m_sw_map.end(); ++sw_node_it) {

		m_ar_mgr_->ARDumpSWSettings(sw_node_it->second);
	}
}

int
OSMParallelPortGroupsCalculator::CalculatePortGroups(
	AdaptiveRoutingAlgorithm *ar_algorithm)
{
	m_max_rank_ = 0;
	m_sw_lid_to_rank_mapping_ = NULL;

	m_lid_mapping_.Clear();
	int rc = ar_algorithm->BuildLidMapping(m_lid_mapping_);
	if (rc) {
		OSM_AR_LOG_RETURN(m_osm_log_, rc);
	}

	/*
	 * Go through all the switches and create their port groups
	 * According to the given algorithm
	 */
	map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it =
	m_sw_db_->m_sw_map.begin();
	for (; sw_node_it != m_sw_db_->m_sw_map.end(); ++sw_node_it) {

		OSM_AR_LOG(m_osm_log_, OSM_LOG_DEBUG,
			   "OSMParallelPortGroupsCalculator AddTask \n");

		AddTaskToThreadPool(
			m_thread_pool_,
			ar_algorithm->GetCalculatePortGroupsTask(sw_node_it->second));
	}

	WaitForTasks();
	OSM_AR_LOG_RETURN(m_osm_log_, 0);
}
