%{
/*
 * Copyright (c) 2013 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#include <stdio.h>
#include <ctype.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_TOPO_PARSER_L_L
#include "osm_topo_parser_y.h"
extern char *__topo_parser_yylval;

static char *trim_whitespace(char *str)
{
	char *end;

	/* Trim leading space */
	while(isspace(*str))
		str++;

	if(*str == '\0')  // All spaces?
		return str;

	/* Trim trailing space */
	end = str + strlen(str) - 1;
	while(end > str && isspace(*end)) end--;

	/*  Write new null terminator */
	*(end+1) = 0;

	return str;
}

%}


%option nounput noinput

%%

\#.*\n				return COMMENT;
[ \t]*topology[ \t]*		return TOPO_START;
[ \t]*end-topology[ \t]*	return TOPO_END;
[ \t]*sw-grp:   	        return SW_GRP;
[ \t]*hca-grp:          	return HCA_GRP;
[ \t]*id:			return ID;
[a-zA-Z0-9'.''_'','' ''\t''\-''|''&''@''('')''/''#''!''?''+''=''[''\]''\/''{''}'';''%''*''^''\\']*                    {
			free(__topo_parser_yylval);
			__topo_parser_yylval=strdup(trim_whitespace(yytext));
			return WORD;
			}
[ \t]*\n                      /* ignore end of line and empty lines*/;
%%
