/*
 * Copyright (c) 2017 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

/*
 * Abstract:
 *      Implementation of AR LAG engine
 */
#define __STDC_FORMAT_MACROS
#include <inttypes.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_UCAST_ARMGR_CPP
#include <opensm/osm_log.h>
#include <opensm/osm_opensm.h>
#include <opensm/osm_subnet.h>
#include <opensm/osm_advanced_routing.h>
#include <opensm/osm_node.h>
#include <opensm/osm_switch.h>

#include <opensm/osm_armgr.hpp>
#include <iostream>
#include <sstream>

extern "C" {

static void * ucast_armgr_construct(osm_subn_t * p_subn, ar_algorithm_t ar_algorithm)
{
	OSMAdaptiveRoutingManager * p_armgr = new OSMAdaptiveRoutingManager(p_subn->p_osm,
									    ar_algorithm);
	if (p_armgr && p_armgr->Construct()) {
		delete p_armgr;
		return NULL;
	}

	return (void *)p_armgr;
}

static int ucast_armgr_route(void *context)
{
	OSMAdaptiveRoutingManager * p_armgr = (OSMAdaptiveRoutingManager *)context;
	return p_armgr->Route();
}

static void ucast_armgr_destroy(void *context)
{
	OSMAdaptiveRoutingManager * p_armgr = (OSMAdaptiveRoutingManager *)context;
	delete p_armgr;
}


int osm_ucast_armgr_lag_setup(osm_adv_routing_engine_t *re, osm_subn_t *p_subn)
{
	re->context = ucast_armgr_construct(p_subn, AR_ALGORITHM_LAG);
	re->destroy = ucast_armgr_destroy;
	re->route = ucast_armgr_route;
	return 0;
}

int osm_ucast_armgr_tree_setup(osm_adv_routing_engine_t *re, osm_subn_t *p_subn)
{
	re->context = ucast_armgr_construct(p_subn, AR_ALGORITHM_TREE);
	re->destroy = ucast_armgr_destroy;
	re->route = ucast_armgr_route;
	return 0;
}

} // extern "C"
