%{
/*
 * Copyright (c) 2016 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_LOG_BYPASS_PARSER_Y_Y
#include <opensm/osm_opensm.h>

#define COMMA ","

#define YYSTYPE char*
extern FILE * __verbose_parser_in;
static struct osm_opensm *p_osm;
char *__verbose_parser_yylval = NULL;


void __verbose_parser_error(const char *str)
{
        OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "%.200s\n", str);
}


int __verbose_parser_wrap()
{
        return 1;
}

int __verbose_parser_parse();
int __verbose_parser_lex();

uint64_t convert_str_64bit(char *str)
{
	char *e = NULL;

	return strtoull(str, &e, 0);
}

#define ATTR_ID_NUM 20
boolean_t validate_addr_id(uint64_t attr_id)
{
	boolean_t found = FALSE;
	uint8_t i = 0;
	ib_net64_t allowed_attr_ids[ATTR_ID_NUM] = {
		IB_MAD_ATTR_PORT_INFO,
		IB_MAD_ATTR_NODE_INFO,
		IB_MAD_ATTR_MLNX_EXTENDED_PORT_INFO,
		IB_MAD_ATTR_SWITCH_INFO,
		IB_MAD_ATTR_GENERAL_INFO,
		IB_MAD_ATTR_NODE_DESC,
		IB_MAD_ATTR_GUID_INFO,
		IB_MAD_ATTR_PATH_RECORD,
		IB_MAD_ATTR_MCMEMBER_RECORD,
		IB_MAD_ATTR_P_KEY_TABLE,
		IB_MAD_ATTR_VIRTUALIZATION_INFO,
		IB_MAD_ATTR_VPORT_INFO,
		IB_MAD_ATTR_VPORT_STATE,
		IB_MAD_ATTR_VNODE_INFO,
		IB_MAD_ATTR_VNODE_DESC,
		IB_MAD_ATTR_ADAPTIVE_ROUTING_INFO,
		IB_MAD_ATTR_ADAPTIVE_ROUTING_LFT,
		IB_MAD_ATTR_ADAPTIVE_ROUTING_GROUP_TABLE,
		IB_MAD_ATTR_ADAPTIVE_ROUTING_GROUP_TABLE_COPY,
		IB_MAD_ATTR_QOS_CONFIG_SL
	};

	while ((!found) && (i < ATTR_ID_NUM)) {
		if ((uint16_t)attr_id == cl_ntoh16(allowed_attr_ids[i]))
			found = TRUE;
		i++;
	}

	return found;
}

void add_attr_id()
{
	uint64_t key;
	char *token;
	char *context = NULL;

	token = strtok_r(__verbose_parser_yylval, COMMA, &context);
	while (token) {
		key  = convert_str_64bit(token);
		if (validate_addr_id(key))
			osm_verbose_bypass_add_attr_id(&p_osm->sm, key);
		else
			OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 2002: "
				"Attr Id 0x%" PRIx64 " is not supported, "
				"Ignoring log verbose bypass request "
				"for this attr id\n", key);
		token = strtok_r(NULL, COMMA, &context);
	}

}

void add_lid()
{
	uint64_t key;
	char *token;
	char *context = NULL;

	token = strtok_r(__verbose_parser_yylval, COMMA, &context);
	while (token) {
		key  = convert_str_64bit(token);
		osm_verbose_bypass_add_lid(&p_osm->sm, key);
		token = strtok_r(NULL, COMMA, &context);
	}

}

void add_guid()
{
	uint64_t key;
	char *token;
	char *context = NULL;

	token = strtok_r(__verbose_parser_yylval, COMMA, &context);
	while (token) {
		key = cl_hton64(convert_str_64bit(token));
		osm_verbose_bypass_add_guid(&p_osm->sm, key);
		token = strtok_r(NULL, COMMA, &context);
	}

}

int parse_verbose_bypass_policy(struct osm_opensm *osm_ptr)
{
	int res = 0;
	p_osm = osm_ptr;
	__verbose_parser_in = NULL;

	osm_verbose_bypass_clear(&p_osm->sm, &p_osm->sm.verbose_bypass);
	p_osm->log.bypass_policy_parsed = FALSE;

	if (!p_osm->subn.opt.verbose_bypass_policy_file) {
		OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "No policy file to parse -"
			" Quit log verbose bypass parser\n");
		goto EXIT;
	}

	__verbose_parser_in = fopen(p_osm->subn.opt.verbose_bypass_policy_file, "r");
	if (!__verbose_parser_in) {
		__verbose_parser_error("ERR 2000: Verbose bypass Policy file specified but not "
				    "found\n");
		res = 1;
		goto EXIT;
	}

        res =  __verbose_parser_parse();
	if (res) {
		__verbose_parser_error("ERR 2001: Verbose bypass policy file parsing "
				    "failed\n");
	} else {
		OSM_LOG(&p_osm->log, OSM_LOG_DEBUG, "Verbose bypass "
			"policy file parsed successfully\n");
		p_osm->log.bypass_policy_parsed = TRUE;
	}
EXIT:
	if (__verbose_parser_in)
	{
		fclose(__verbose_parser_in);
		__verbose_parser_in = NULL;
	}
	return res;
}

%}

%token COMMENT
%token WORD
%token ATTR_ID
%token GUID
%token LID

%%

verbose_bypass_policy: rules;

rules: /* empty */
        |  rules rule
        ;

comment: COMMENT
	{
	}
	;

rule:
	attr_id
	|guid
	|lid
	|comment
	|word
        ;


word: WORD
	{
		OSM_LOG(&p_osm->log, OSM_LOG_ERROR, "ERR 2003: "
			"Ignoring unrecognized phrase in log verbose bypass "
			"policy file (%s)\n", __verbose_parser_yylval);
		free(__verbose_parser_yylval);
	}
	;

attr_id: ATTR_ID WORD
	{
		add_attr_id();
		free(__verbose_parser_yylval);
	}
	;

guid: GUID WORD
	{
		add_guid();
		free(__verbose_parser_yylval);
	}
	;

lid: LID WORD
	{
		add_lid();
		free(__verbose_parser_yylval);
	}
	;
